@@ -80,6 +80,11 @@ SimpleCurlGet::GetStatus SimpleCurlGet::perform_start(const std::string &url) {
     if (rv != CURLE_OK) {
         throw CurlException("Failed to set CURLOPT_FOLLOWLOCATION.");
     }
+    // Disable signal handling to avoid issues in multi-threaded contexts.
+    rv = curl_easy_setopt(m_curl.get(), CURLOPT_NOSIGNAL, 1L);
+    if (rv != CURLE_OK) {
+        throw CurlException("Failed to set CURLOPT_NOSIGNAL.");
+    }
 
     auto ca_file = configurer::Configuration::get_tls_ca_file();
     if (!ca_file.empty()) {
@@ -527,8 +532,6 @@ std::string rs256_from_coords(const std::string &e_str,
         throw UnsupportedKeyException("Failed to serialize RSA public key");
     }
 #endif
-    e_bignum.release();
-    n_bignum.release();
 
     char *mem_data;
     size_t mem_len = BIO_get_mem_data(pubkey_bio.get(), &mem_data);
@@ -576,6 +579,15 @@ std::string normalize_absolute_path(const std::string &path) {
 }
 
 } // namespace
+  //
+static std::unordered_map<std::string, jwt::claim>
+json_to_claim_map(const picojson::object &json) {
+    std::unordered_map<std::string, jwt::claim> m;
+    for (const auto &[name, value] : json) {
+        m.emplace(name, jwt::claim(value));
+    }
+    return m; // nvro
+}
 
 void SciToken::deserialize(const std::string &data,
                            const std::vector<std::string> allowed_issuers) {
@@ -588,7 +600,7 @@ void SciToken::deserialize(const std::string &data,
     val.verify(*m_decoded);
 
     // Set all the claims
-    m_claims = m_decoded->get_payload_claims();
+    m_claims = json_to_claim_map(m_decoded->get_payload_json());
 
     // Copy over the profile
     m_profile = val.get_profile();
@@ -616,7 +628,7 @@ SciToken::deserialize_continue(std::unique_ptr<SciTokenAsyncStatus> status) {
     // Check if the status is completed (verification is complete)
     if (status->m_status->m_done) {
         // Set all the claims
-        m_claims = m_decoded->get_payload_claims();
+        m_claims = json_to_claim_map(m_decoded->get_payload_json());
 
         // Copy over the profile
         m_profile = status->m_validator->get_profile();
@@ -625,7 +637,7 @@ SciToken::deserialize_continue(std::unique_ptr<SciTokenAsyncStatus> status) {
             std::move(status->m_status));
         if (status->m_status->m_done) {
             // Set all the claims
-            m_claims = m_decoded->get_payload_claims();
+            m_claims = json_to_claim_map(m_decoded->get_payload_json());
 
             // Copy over the profile
             m_profile = status->m_validator->get_profile();
@@ -722,12 +734,12 @@ std::unique_ptr<AsyncStatus> Validator::get_public_keys_from_web_continue(
         auto metadata = std::string(buffer, len);
         picojson::value json_obj;
         auto err = picojson::parse(json_obj, metadata);
-        status->m_cget.reset();
         if (!err.empty()) {
             throw JsonException("JSON parse failure when downloading from the "
                                 " public key URL " +
                                 status->m_cget->get_url() + ": " + err);
         }
+        status->m_cget.reset();
 
         auto now = std::time(NULL);
         // TODO: take expiration time from the cache-control header in the
@@ -951,12 +963,12 @@ bool scitokens::Validator::store_public_ec_key(const std::string &issuer,
             "Unable to get OpenSSL public key parameters");
     }
 
-    void *buf = NULL;
-    size_t buf_len, max_len = 256;
+    const void *buf = NULL;
+    size_t buf_len;
     OSSL_PARAM *p = OSSL_PARAM_locate(params, "pub");
-    if (!p || !OSSL_PARAM_get_octet_string(p, &buf, max_len, &buf_len) ||
+    if (!p || !OSSL_PARAM_get_octet_string_ptr(p, &buf, &buf_len) ||
         !EC_POINT_oct2point(ec_group.get(), q_point.get(),
-                            static_cast<unsigned char *>(buf), buf_len,
+                            static_cast<const unsigned char *>(buf), buf_len,
                             nullptr)) {
         throw UnsupportedKeyException(
             "Failed to to set OpenSSL EC point with public key information");
