@@ -156,7 +156,8 @@ class SciTokenKey {
         : m_kid(key_id), m_name(algorithm), m_public(public_contents),
           m_private(private_contents) {}
 
-    std::string serialize(jwt::builder<jwt::traits::kazuho_picojson> &builder) {
+    std::string serialize(jwt::builder<jwt::default_clock,
+                                       jwt::traits::kazuho_picojson> &builder) {
         if (m_kid != "none") {
             builder.set_key_id(m_kid);
         }
@@ -327,7 +328,7 @@ class SciToken {
     void set_lifetime(int lifetime) { m_lifetime = lifetime; }
 
     std::string serialize() {
-        jwt::builder<jwt::traits::kazuho_picojson> builder(jwt::create());
+        auto builder(jwt::create());
 
         if (!m_issuer_set) {
             throw MissingIssuerException();
@@ -486,8 +487,10 @@ class Validator {
                 }
             }
             if (!permitted) {
+                std::string safe_issuer = format_issuer_for_error(jwt);
                 throw JWTVerificationException(
-                    "Token issuer is not in list of allowed issuers.");
+                    "Token issuer " + safe_issuer +
+                    " is not in list of allowed issuers.");
             }
         }
 
@@ -614,7 +617,7 @@ class Validator {
             must_verify_everything = m_validate_all_claims;
         }
 
-        auto claims = jwt.get_payload_claims();
+        auto claims = jwt.get_payload_json();
         for (const auto &claim_pair : claims) {
             if (claim_pair.first == "iat" || claim_pair.first == "nbf" ||
                 claim_pair.first == "exp" || claim_pair.first == "ver") {
@@ -772,6 +775,33 @@ class Validator {
                                   const picojson::value &keys,
                                   int64_t next_update, int64_t expires);
 
+    /**
+     * Safely format an issuer for error messages.
+     * Serializes the issuer claim back to JSON format and limits the size
+     * to prevent malicious issuers from causing problems in error output.
+     */
+    static std::string format_issuer_for_error(
+        const jwt::decoded_jwt<jwt::traits::kazuho_picojson> &jwt) {
+        try {
+            if (!jwt.has_payload_claim("iss")) {
+                return "<missing issuer>";
+            }
+            // Get the raw claim and serialize it back to JSON
+            const auto &claim = jwt.get_payload_claim("iss");
+            std::string serialized = claim.to_json().serialize();
+            // Limit the size to prevent abuse
+            const size_t max_issuer_length = 256;
+            if (serialized.length() > max_issuer_length) {
+                serialized =
+                    serialized.substr(0, max_issuer_length - 3) + "...";
+            }
+            return serialized;
+        } catch (...) {
+            // If anything goes wrong, return a safe fallback
+            return "<invalid issuer>";
+        }
+    }
+
     bool m_validate_all_claims{true};
     SciToken::Profile m_profile{SciToken::Profile::COMPAT};
     SciToken::Profile m_validate_profile{SciToken::Profile::COMPAT};
