@@ -131,10 +131,11 @@ TEST_F(SerializeTest, VerifyTest) {
     ASSERT_TRUE(rv == 0) << err_msg;
     std::unique_ptr<char, decltype(&free)> value_ptr(value, free);
     EXPECT_STREQ(value, "https://demo.scitokens.org/gtest");
-
     value_ptr.reset();
+
     rv = scitoken_get_claim_string(m_read_token.get(), "doesnotexist", &value,
                                    &err_msg);
+    free(err_msg);
     EXPECT_FALSE(rv == 0);
 }
 
@@ -154,6 +155,7 @@ TEST_F(SerializeTest, TestStringList) {
     EXPECT_STREQ(value[1], "group1");
 
     EXPECT_TRUE(value[2] == nullptr);
+    scitoken_free_string_list(value);
 }
 
 TEST_F(SerializeTest, VerifyWLCGTest) {
@@ -181,6 +183,8 @@ TEST_F(SerializeTest, VerifyWLCGTest) {
 
     value_ptr.reset();
     rv = scitoken_get_claim_string(m_read_token.get(), "ver", &value, &err_msg);
+    free(err_msg);
+    err_msg = nullptr;
     EXPECT_FALSE(rv == 0);
 
     // Accepts only a WLCG token
@@ -195,6 +199,7 @@ TEST_F(SerializeTest, VerifyWLCGTest) {
                                      SciTokenProfile::SCITOKENS_1_0);
     rv = scitoken_deserialize_v2(token_value, m_read_token.get(), nullptr,
                                  &err_msg);
+    free(err_msg);
     ASSERT_FALSE(rv == 0);
 }
 
@@ -223,6 +228,7 @@ TEST_F(SerializeTest, FailVerifyToken) {
 
     // Should fail; we signed it with the wrong public key.
     rv = scitoken_deserialize_v2(value, m_read_token.get(), nullptr, &err_msg);
+    free(err_msg);
     EXPECT_FALSE(rv == 0);
 }
 
@@ -254,6 +260,7 @@ TEST_F(SerializeTest, VerifyATJWTTest) {
                                      SciTokenProfile::SCITOKENS_2_0);
     rv = scitoken_deserialize_v2(token_value, m_read_token.get(), nullptr,
                                  &err_msg);
+    free(err_msg);
     ASSERT_FALSE(rv == 0);
 }
 
@@ -278,6 +285,7 @@ TEST_F(SerializeTest, FailVerifyATJWTTest) {
                                      SciTokenProfile::AT_JWT);
     rv = scitoken_deserialize_v2(token_value, m_read_token.get(), nullptr,
                                  &err_msg);
+    free(err_msg);
     ASSERT_FALSE(rv == 0);
 }
 
@@ -315,12 +323,15 @@ TEST_F(SerializeTest, EnforcerTest) {
     rv = scitoken_deserialize_v2(token_value, m_read_token.get(), nullptr,
                                  &err_msg);
     ASSERT_TRUE(rv == 0) << err_msg;
+    free(err_msg);
 
     rv = enforcer_test(enforcer, m_read_token.get(), &acl, &err_msg);
     ASSERT_STREQ(
         err_msg,
         "token verification failed: 'scope' claim verification failed.");
     ASSERT_TRUE(rv == -1) << err_msg;
+    free(err_msg);
+    enforcer_destroy(enforcer);
 }
 
 TEST_F(SerializeTest, EnforcerScopeTest) {
@@ -347,6 +358,7 @@ TEST_F(SerializeTest, EnforcerScopeTest) {
 
     rv = scitoken_deserialize_v2(token_value, m_read_token.get(), nullptr,
                                  &err_msg);
+    free(token_value);
     ASSERT_TRUE(rv == 0) << err_msg;
 
     Acl *acls;
@@ -366,6 +378,8 @@ TEST_F(SerializeTest, EnforcerScopeTest) {
             ASSERT_STREQ(resource, "/");
         }
     }
+    enforcer_acl_free(acls);
+    enforcer_destroy(enforcer);
     ASSERT_TRUE(found_read);
     ASSERT_TRUE(found_write);
 }
@@ -395,6 +409,7 @@ TEST_F(SerializeTest, DeserializeAsyncTest) {
         rv = scitoken_deserialize_continue(&scitoken, &status, &err_msg);
         ASSERT_TRUE(rv == 0) << err_msg;
     }
+    scitoken_destroy(scitoken);
 }
 
 TEST_F(SerializeTest, FailDeserializeAsyncTest) {
@@ -430,11 +445,15 @@ TEST_F(SerializeTest, FailDeserializeAsyncTest) {
     rv = scitoken_deserialize_start(value, &scitoken, nullptr, &status,
                                     &err_msg);
     EXPECT_FALSE(rv == 0) << err_msg;
+    free(err_msg);
+    err_msg = nullptr;
 
     // Accepts only an at+jwt token, should fail with COMPAT token
     while (rv == 0 && status) {
         rv = scitoken_deserialize_continue(&scitoken, &status, &err_msg);
         EXPECT_FALSE(rv == 0) << err_msg;
+        free(err_msg);
+        err_msg = nullptr;
     }
 }
 
@@ -452,6 +471,7 @@ TEST_F(SerializeTest, ExplicitTime) {
 
     rv = scitoken_deserialize_v2(token_value, m_read_token.get(), nullptr,
                                  &err_msg);
+    free(token_value);
     ASSERT_TRUE(rv == 0) << err_msg;
 
     auto enforcer = enforcer_create("https://demo.scitokens.org/gtest",
@@ -461,22 +481,62 @@ TEST_F(SerializeTest, ExplicitTime) {
     rv = enforcer_generate_acls(enforcer, m_read_token.get(), &acls, &err_msg);
     ASSERT_TRUE(rv == 0) << err_msg;
     ASSERT_TRUE(acls != nullptr);
+    enforcer_acl_free(acls);
 
     enforcer_set_time(enforcer, time(NULL), &err_msg);
     rv = enforcer_generate_acls(enforcer, m_read_token.get(), &acls, &err_msg);
     ASSERT_TRUE(rv == 0) << err_msg;
 
     enforcer_set_time(enforcer, time(NULL) + 100, &err_msg);
+    enforcer_acl_free(acls);
     rv = enforcer_generate_acls(enforcer, m_read_token.get(), &acls, &err_msg);
+    free(err_msg);
+    err_msg = nullptr;
     ASSERT_FALSE(rv == 0);
 
     enforcer_set_time(enforcer, time(NULL) - 100, &err_msg);
     rv = enforcer_generate_acls(enforcer, m_read_token.get(), &acls, &err_msg);
     ASSERT_FALSE(rv == 0);
+    free(err_msg);
 
     enforcer_destroy(enforcer);
 }
 
+TEST_F(SerializeTest, GetExpirationErrorHandling) {
+    char *err_msg = nullptr;
+    
+    // Test NULL token handling
+    long long expiry;
+    auto rv = scitoken_get_expiration(nullptr, &expiry, &err_msg);
+    ASSERT_FALSE(rv == 0);
+    ASSERT_TRUE(err_msg != nullptr);
+    EXPECT_STREQ(err_msg, "Token cannot be NULL");
+    free(err_msg);
+    err_msg = nullptr;
+    
+    // Test NULL expiry parameter handling  
+    rv = scitoken_get_expiration(m_token.get(), nullptr, &err_msg);
+    ASSERT_FALSE(rv == 0);
+    ASSERT_TRUE(err_msg != nullptr);
+    EXPECT_STREQ(err_msg, "Expiry output parameter cannot be NULL");
+    free(err_msg);
+    err_msg = nullptr;
+    
+    // Test normal operation works
+    char *token_value = nullptr;
+    rv = scitoken_serialize(m_token.get(), &token_value, &err_msg);
+    ASSERT_TRUE(rv == 0) << err_msg;
+    
+    rv = scitoken_deserialize_v2(token_value, m_read_token.get(), nullptr, &err_msg);
+    ASSERT_TRUE(rv == 0) << err_msg;
+    
+    rv = scitoken_get_expiration(m_read_token.get(), &expiry, &err_msg);
+    ASSERT_TRUE(rv == 0) << err_msg;
+    ASSERT_TRUE(expiry > 0);
+    
+    free(token_value);
+}
+
 class SerializeNoKidTest : public ::testing::Test {
   protected:
     void SetUp() override {
@@ -554,6 +614,7 @@ TEST_F(SerializeNoKidTest, VerifyATJWTTest) {
                                      SciTokenProfile::SCITOKENS_2_0);
     rv = scitoken_deserialize_v2(token_value, m_read_token.get(), nullptr,
                                  &err_msg);
+    free(err_msg);
     ASSERT_FALSE(rv == 0);
 }
 
@@ -611,6 +672,7 @@ TEST_F(KeycacheTest, RefreshInvalid) {
     auto rv =
         keycache_refresh_jwks("https://demo.scitokens.org/invalid", &err_msg);
     ASSERT_FALSE(rv == 0);
+    free(err_msg);
 
     rv = keycache_get_cached_jwks("https://demo.scitokens.org/invalid", &jwks,
                                   &err_msg);
@@ -703,10 +765,13 @@ TEST_F(KeycacheTest, InvalidConfigKeyTest) {
     std::string key = "invalid key";
     auto rv =
         scitoken_config_set_int(key.c_str(), new_update_interval, &err_msg);
+    free(err_msg);
+    err_msg = nullptr;
     ASSERT_FALSE(rv == 0);
 
     const char *key2 = nullptr;
     rv = scitoken_config_set_int(key2, new_update_interval, &err_msg);
+    free(err_msg);
     ASSERT_FALSE(rv == 0);
 }
 
@@ -740,6 +805,7 @@ TEST_F(KeycacheTest, SetInvalidUpdateTest) {
     std::string key = "keycache.update_interval_s";
     auto rv =
         scitoken_config_set_int(key.c_str(), new_update_interval, &err_msg);
+    free(err_msg);
     ASSERT_FALSE(rv == 0);
 }
 
@@ -749,6 +815,7 @@ TEST_F(KeycacheTest, SetInvalidExpirationTest) {
     std::string key = "keycache.expiration_interval_s";
     auto rv =
         scitoken_config_set_int(key.c_str(), new_expiration_interval, &err_msg);
+    free(err_msg);
     ASSERT_FALSE(rv == 0);
 }
 
@@ -774,6 +841,106 @@ TEST_F(KeycacheTest, RefreshExpiredTest) {
     EXPECT_EQ(jwks_str, "{\"keys\": []}");
 }
 
+class IssuerSecurityTest : public ::testing::Test {
+  protected:
+    void SetUp() override {
+        char *err_msg = nullptr;
+        m_key = KeyPtr(
+            scitoken_key_create("1", "ES256", ec_public, ec_private, &err_msg),
+            scitoken_key_destroy);
+        ASSERT_TRUE(m_key.get() != nullptr) << err_msg;
+
+        m_token = TokenPtr(scitoken_create(m_key.get()), scitoken_destroy);
+        ASSERT_TRUE(m_token.get() != nullptr);
+
+        // Store public key for verification
+        auto rv = scitoken_store_public_ec_key(
+            "https://demo.scitokens.org/gtest", "1", ec_public, &err_msg);
+        ASSERT_TRUE(rv == 0) << err_msg;
+
+        scitoken_set_lifetime(m_token.get(), 60);
+
+        m_read_token.reset(scitoken_create(nullptr));
+        ASSERT_TRUE(m_read_token.get() != nullptr);
+    }
+
+    using KeyPtr = std::unique_ptr<void, decltype(&scitoken_key_destroy)>;
+    KeyPtr m_key{nullptr, scitoken_key_destroy};
+
+    using TokenPtr = std::unique_ptr<void, decltype(&scitoken_destroy)>;
+    TokenPtr m_token{nullptr, scitoken_destroy};
+
+    TokenPtr m_read_token{nullptr, scitoken_destroy};
+};
+
+TEST_F(IssuerSecurityTest, LongIssuerTruncation) {
+    char *err_msg = nullptr;
+
+    // Create a very long issuer (1000 characters)
+    std::string very_long_issuer(1000, 'A');
+    auto rv = scitoken_set_claim_string(m_token.get(), "iss",
+                                        very_long_issuer.c_str(), &err_msg);
+    ASSERT_TRUE(rv == 0) << err_msg;
+
+    char *token_value = nullptr;
+    rv = scitoken_serialize(m_token.get(), &token_value, &err_msg);
+    ASSERT_TRUE(rv == 0) << err_msg;
+    std::unique_ptr<char, decltype(&free)> token_value_ptr(token_value, free);
+
+    // Try to verify with a restricted issuer list to trigger error
+    const char *allowed_issuers[] = {"https://good.issuer.com", nullptr};
+    rv = scitoken_deserialize_v2(token_value, m_read_token.get(),
+                                 allowed_issuers, &err_msg);
+
+    // Should fail
+    ASSERT_FALSE(rv == 0);
+    ASSERT_TRUE(err_msg != nullptr);
+    std::string error_message(err_msg);
+    std::unique_ptr<char, decltype(&free)> err_msg_ptr(err_msg, free);
+    // Error message should be reasonable length (< 400 chars)
+    EXPECT_LT(error_message.length(), 400);
+    // Should contain expected error text
+    EXPECT_NE(error_message.find("is not in list of allowed issuers"),
+              std::string::npos);
+
+    // Should contain truncated issuer with ellipsis
+    EXPECT_NE(error_message.find("..."), std::string::npos);
+}
+
+TEST_F(IssuerSecurityTest, SpecialCharacterIssuer) {
+    char *err_msg = nullptr;
+
+    // Create an issuer with special characters and control chars
+    std::string special_issuer = "https://bad.com/\"\n\t\r\x01\x1f";
+    auto rv = scitoken_set_claim_string(m_token.get(), "iss",
+                                        special_issuer.c_str(), &err_msg);
+    ASSERT_TRUE(rv == 0) << err_msg;
+
+    char *token_value = nullptr;
+    rv = scitoken_serialize(m_token.get(), &token_value, &err_msg);
+    ASSERT_TRUE(rv == 0) << err_msg;
+    std::unique_ptr<char, decltype(&free)> token_value_ptr(token_value, free);
+
+    // Try to verify with a restricted issuer list to trigger error
+    const char *allowed_issuers[] = {"https://good.issuer.com", nullptr};
+    rv = scitoken_deserialize_v2(token_value, m_read_token.get(),
+                                 allowed_issuers, &err_msg);
+
+    // Should fail
+    ASSERT_FALSE(rv == 0);
+    ASSERT_TRUE(err_msg != nullptr);
+    std::string error_message(err_msg);
+    std::unique_ptr<char, decltype(&free)> err_msg_ptr(err_msg, free);
+    // Error message should be reasonable length
+    EXPECT_LT(error_message.length(), 300);
+    // Should contain expected error text
+    EXPECT_NE(error_message.find("is not in list of allowed issuers"),
+              std::string::npos);
+
+    // Should contain properly escaped JSON (with quotes)
+    EXPECT_NE(error_message.find("\""), std::string::npos);
+}
+
 int main(int argc, char **argv) {
     ::testing::InitGoogleTest(&argc, argv);
     return RUN_ALL_TESTS();
